@@ -13,6 +13,7 @@ module Web.Cookie
     , setCookieHttpOnly
     , setCookieSecure
     , setCookieSameSite
+    , setCookiePartitioned
     , SameSiteOption
     , sameSiteLax
     , sameSiteStrict
@@ -55,7 +56,7 @@ import Control.Arrow (first, (***))
 import Data.Text (Text)
 import Data.Text.Encoding (encodeUtf8Builder, decodeUtf8With)
 import Data.Text.Encoding.Error (lenientDecode)
-import Data.Maybe (isJust)
+import Data.Maybe (isJust, fromMaybe, listToMaybe)
 import Data.Default.Class (Default (def))
 import Control.DeepSeq (NFData (rnf))
 
@@ -73,25 +74,46 @@ renderCookiesText = renderCookiesBuilder . map (encodeUtf8Builder *** encodeUtf8
 
 type Cookies = [(S.ByteString, S.ByteString)]
 
+semicolon :: Word8
+semicolon = 59
+
+equalsSign :: Word8
+equalsSign = 61
+
+space :: Word8
+space = 32
+
+doubleQuote :: Word8
+doubleQuote = 34
+
 -- | Decode the value of a \"Cookie\" request header into key/value pairs.
 parseCookies :: S.ByteString -> Cookies
 parseCookies s
   | S.null s = []
   | otherwise =
-    let (x, y) = breakDiscard 59 s -- semicolon
+    let (x, y) = breakDiscard semicolon s
      in parseCookie x : parseCookies y
 
 parseCookie :: S.ByteString -> (S.ByteString, S.ByteString)
 parseCookie s =
-    let (key, value) = breakDiscard 61 s -- equals sign
-        key' = S.dropWhile (== 32) key -- space
-     in (key', value)
+    let (key, value) = breakDiscard equalsSign s
+        key' = S.dropWhile (== space) key
+        value' = dropEnds doubleQuote value
+     in (key', value')
 
 breakDiscard :: Word8 -> S.ByteString -> (S.ByteString, S.ByteString)
 breakDiscard w s =
     let (x, y) = S.break (== w) s
      in (x, S.drop 1 y)
 
+dropEnds :: Word8 -> S.ByteString -> S.ByteString
+dropEnds w s =
+    case S.unsnoc s of
+        Just (s', w') | w' == w -> case S.uncons s' of
+            Just (w'', s'') | w'' == w -> s''
+            _ -> s
+        _ -> s
+
 type CookieBuilder = (Builder, Builder)
 
 renderCookiesBuilder :: [CookieBuilder] -> Builder
@@ -136,6 +158,7 @@ data SetCookie = SetCookie
     , setCookieHttpOnly :: Bool -- ^ Marks the cookie as "HTTP only", i.e. not accessible from Javascript. Default value: @False@
     , setCookieSecure :: Bool -- ^ Instructs the browser to only send the cookie over HTTPS. Default value: @False@
     , setCookieSameSite :: Maybe SameSiteOption -- ^ The "same site" policy of the cookie, i.e. whether it should be sent with cross-site requests. Default value: @Nothing@
+    , setCookiePartitioned :: Bool -- ^ Cookies marked Partitioned are double-keyed: by the origin that sets them and the origin of the top-level page. Default value: @False@
     }
     deriving (Eq, Show)
 
@@ -164,7 +187,7 @@ sameSiteNone :: SameSiteOption
 sameSiteNone = None
 
 instance NFData SetCookie where
-    rnf (SetCookie a b c d e f g h i) =
+    rnf (SetCookie a b c d e f g h i j) =
         a `seq`
         b `seq`
         rnfMBS c `seq`
@@ -173,7 +196,8 @@ instance NFData SetCookie where
         rnfMBS f `seq`
         rnf g `seq`
         rnf h `seq`
-        rnf i
+        rnf i `seq`
+        rnf j
       where
         -- For backwards compatibility
         rnfMBS Nothing = ()
@@ -197,6 +221,7 @@ defaultSetCookie = SetCookie
     , setCookieHttpOnly = False
     , setCookieSecure   = False
     , setCookieSameSite = Nothing
+    , setCookiePartitioned = False
     }
 
 renderSetCookie :: SetCookie -> Builder
@@ -214,7 +239,7 @@ renderSetCookie sc = mconcat
                   byteString (formatCookieExpires e)
     , case setCookieMaxAge sc of
         Nothing -> mempty
-        Just ma -> byteStringCopy"; Max-Age=" `mappend`
+        Just ma -> byteStringCopy "; Max-Age=" `mappend`
                    byteString (formatCookieMaxAge ma)
     , case setCookieDomain sc of
         Nothing -> mempty
@@ -231,6 +256,9 @@ renderSetCookie sc = mconcat
         Just Lax -> byteStringCopy "; SameSite=Lax"
         Just Strict -> byteStringCopy "; SameSite=Strict"
         Just None -> byteStringCopy "; SameSite=None"
+    , if setCookiePartitioned sc
+        then byteStringCopy "; Partitioned"
+        else mempty
     ]
 
 -- | @since 0.4.6
@@ -240,7 +268,7 @@ renderSetCookieBS = L.toStrict . toLazyByteString . renderSetCookie
 parseSetCookie :: S.ByteString -> SetCookie
 parseSetCookie a = SetCookie
     { setCookieName = name
-    , setCookieValue = value
+    , setCookieValue = dropEnds doubleQuote value
     , setCookiePath = lookup "path" flags
     , setCookieExpires =
         lookup "expires" flags >>= parseCookieExpires
@@ -254,13 +282,14 @@ parseSetCookie a = SetCookie
         Just "Strict" -> Just Strict
         Just "None" -> Just None
         _ -> Nothing
+    , setCookiePartitioned = isJust $ lookup "partitioned" flags
     }
   where
-    pairs = map (parsePair . dropSpace) $ S.split 59 a ++ [S8.empty] -- 59 = semicolon
-    (name, value) = head pairs
-    flags = map (first (S8.map toLower)) $ tail pairs
-    parsePair = breakDiscard 61 -- equals sign
-    dropSpace = S.dropWhile (== 32) -- space
+    pairs = map (parsePair . dropSpace) $ S.split semicolon a
+    (name, value) = fromMaybe mempty $ listToMaybe pairs
+    flags = map (first (S8.map toLower)) $ drop 1 pairs
+    parsePair = breakDiscard equalsSign
+    dropSpace = S.dropWhile (== space)
 
 expiresFormat :: String
 expiresFormat = "%a, %d-%b-%Y %X GMT"
