@@ -1,18 +1,19 @@
-import Test.Tasty (defaultMain, testGroup)
+{-# OPTIONS_GHC -Wno-orphans #-}
+module Main where
+
+import Test.Tasty (defaultMain, testGroup, TestTree)
 import Test.Tasty.QuickCheck (testProperty)
 import Test.Tasty.HUnit (testCase)
 import Test.QuickCheck
 import Test.HUnit ((@=?), Assertion)
 
 import Web.Cookie
-import Data.ByteString.Builder (Builder, word8, toLazyByteString)
+import Data.ByteString.Builder (Builder, toLazyByteString)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 import Data.Word (Word8)
-import Data.Monoid (mconcat)
 import Control.Arrow ((***))
-import Control.Applicative ((<$>), (<*>))
 import Data.Time (UTCTime (UTCTime), toGregorian)
 import qualified Data.Text as T
 
@@ -22,6 +23,8 @@ main = defaultMain $ testGroup "cookie"
     , testProperty "parse/render SetCookie" propParseRenderSetCookie
     , testProperty "parse/render cookies text" propParseRenderCookiesText
     , testCase "parseCookies" caseParseCookies
+    , testCase "parseQuotedCookies" caseParseQuotedCookies
+    , testCase "parseQuotedSetCookie" caseParseQuotedSetCookie
     , twoDigit 24 2024
     , twoDigit 69 2069
     , twoDigit 70 1970
@@ -52,7 +55,7 @@ instance Show Char' where
     show (Char' w) = [toEnum $ fromEnum w]
     showList = (++) . show . concatMap show
 instance Arbitrary Char' where
-    arbitrary = fmap (Char' . toEnum) $ choose (62, 125)
+    arbitrary = Char' . toEnum <$> choose (62, 125)
 newtype SameSiteOption' = SameSiteOption' { unSameSiteOption' :: SameSiteOption }
 instance Arbitrary SameSiteOption' where
   arbitrary = fmap SameSiteOption' (elements [sameSiteLax, sameSiteStrict, sameSiteNone])
@@ -72,6 +75,7 @@ instance Arbitrary SetCookie where
         httponly <- arbitrary
         secure <- arbitrary
         sameSite <- fmap (fmap unSameSiteOption') arbitrary
+        partitioned <- arbitrary
         return def
             { setCookieName = name
             , setCookieValue = value
@@ -81,6 +85,7 @@ instance Arbitrary SetCookie where
             , setCookieHttpOnly = httponly
             , setCookieSecure = secure
             , setCookieSameSite = sameSite
+            , setCookiePartitioned = partitioned
             }
 
 caseParseCookies :: Assertion
@@ -89,16 +94,20 @@ caseParseCookies = do
         expected = [("a", "a1"), ("b", "b2"), ("c", "c3")]
     map (S8.pack *** S8.pack) expected @=? parseCookies input
 
+-- TODO: Use `Year` from Data.Time when we'll remove support for GHC <9.2
+type Year = Integer
+
 -- Tests for two digit years, see:
 --
 -- https://github.com/snoyberg/cookie/issues/5
+twoDigit :: Year -> Year -> TestTree
 twoDigit x y =
     testCase ("year " ++ show x) (y @=? year)
   where
     (year, _, _) = toGregorian day
     day =
         case setCookieExpires sc of
-            Just (UTCTime day _) -> day
+            Just (UTCTime day' _) -> day'
             Nothing -> error $ "setCookieExpires == Nothing for: " ++ show str
     sc = parseSetCookie str
     str = S8.pack $ concat
@@ -106,3 +115,17 @@ twoDigit x y =
         , show x
         , " 04:52:08 GMT"
         ]
+
+caseParseQuotedCookies :: Assertion
+caseParseQuotedCookies = do
+    let input = S8.pack "a=\"a1\";b=\"b2\"; c=\"c3\""
+        expected = [("a", "a1"), ("b", "b2"), ("c", "c3")]
+    map (S8.pack *** S8.pack) expected @=? parseCookies input
+
+caseParseQuotedSetCookie :: Assertion
+caseParseQuotedSetCookie = do
+    let input = S8.pack "a=\"a1\""
+        result = parseSetCookie input
+        resultNameAndValue = (setCookieName result, setCookieValue result)
+        expected = (S8.pack "a", S8.pack "a1")
+    expected @=? resultNameAndValue
